"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _gulp = _interopRequireDefault(require("gulp"));

var _gulpReplace = _interopRequireDefault(require("gulp-replace"));

var _gulpRename = _interopRequireDefault(require("gulp-rename"));

var _del = _interopRequireDefault(require("del"));

var _lerna = _interopRequireDefault(require("./lerna"));

var create = function create(_ref) {
  var root = _ref.root,
      _ref$defaultTask = _ref.defaultTask,
      defaultTask = _ref$defaultTask === void 0 ? 'build' : _ref$defaultTask,
      lernaConfig = _ref.lernaConfig,
      tsConfig = _ref.tsConfig,
      _ref$replaceFind = _ref.replaceFind,
      replaceFind = _ref$replaceFind === void 0 ? /(import|export)(.*)from '@znemz\/(.*)\/src\/(.*)';/g : _ref$replaceFind,
      _ref$replaceStr = _ref.replaceStr,
      replaceStr = _ref$replaceStr === void 0 ? "$1$2from '@znemz/$3/lib/$4';" : _ref$replaceStr,
      _ref$tsFilter = _ref.tsFilter,
      tsFilter = _ref$tsFilter === void 0 ? ['ts', 'tsx', 'd.ts'] : _ref$tsFilter;

  if (!root) {
    throw new Error('root path {root} must be defined.');
  }

  var gulpBuiltTask = (0, _lerna["default"])({
    root: root,
    lernaConfig: lernaConfig,
    replaceFind: replaceFind,
    replaceStr: replaceStr,
    tsFilter: tsFilter,
    tsConfig: tsConfig
  });
  gulpBuiltTask('build');
  gulpBuiltTask('build:ts', {
    doTs: true,
    doBabel: false,
    doTypes: false
  });

  _gulp["default"].task('docs:clean', function () {
    return (0, _del["default"])(['docs/**/*.md', 'docs/packages']);
  });

  _gulp["default"].task('docs:assets', function () {
    return _gulp["default"].src(['docs/images/**/*', 'docs/*.ico'], {
      base: './docs'
    }).pipe(_gulp["default"].dest('public')) // deal w/ routing bugs and satisfy assets there as well
    .pipe(_gulp["default"].dest('public/docs'));
  });

  var docsSource = ['**/*.md', '**/*.mdx', '!**/docs/**', '!**/node_modules/**', '!node_modules'];

  _gulp["default"].task('docs', _gulp["default"].series('docs:clean', function () {
    return _gulp["default"].src(docsSource).pipe((0, _gulpReplace["default"])(/\/index.md/g, '')) // fix links
    .pipe((0, _gulpRename["default"])(function (filePath) {
      if (filePath.basename === 'README') {
        filePath.basename = 'index';

        if (filePath.dirname === '.') {
          // we're at root README
          filePath.basename = 'docs';
        }
      }
    })).pipe(_gulp["default"].dest('docs'));
  }));

  _gulp["default"].task('docs:watch', _gulp["default"].series('docs', function () {
    _gulp["default"].watch(docsSource, _gulp["default"].series('docs'));
  }));

  _gulp["default"].task('default', _gulp["default"].series('clean', _gulp["default"].parallel(defaultTask)));
  /*
  This is used fo CI only to make sure typescript builds and there are no errors.
  This is to make up for the fact that babel-typescript is much more forgiving.
  */


  _gulp["default"].task('typescript', _gulp["default"].series('clean', 'build:ts', 'clean'));

  _gulp["default"].task('tsc', _gulp["default"].series('build:ts'));
};

var _default = create;
exports["default"] = _default;