"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.flattenTasks = exports.gulpBuildTask = exports.getBuildOptions = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _gulp = _interopRequireDefault(require("gulp"));

var _lodash = require("lodash");

var _path = _interopRequireDefault(require("path"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var debug = require('../../debug').spawn('lerna:buildTasks');

var getBuildOptions = function getBuildOptions(_ref) {
  var root = _ref.root,
      pack = _ref.pack,
      doBabel = _ref.doBabel,
      doTs = _ref.doTs,
      doTypes = _ref.doTypes;
  var dbg = debug.spawn('getBuildOptions');

  var pkgPath = _path["default"].join(root, 'packages', pack, 'package.json');

  dbg(function () {
    return pkgPath;
  });

  var pkg = require(pkgPath);

  dbg(function () {
    return 'succes';
  });
  var pkgOpts = (0, _lodash.get)(pkg, ['gulp', 'opts'], {});
  return _objectSpread({
    doBabel: doBabel,
    doTs: doTs,
    doTypes: doTypes
  }, pkgOpts);
};

exports.getBuildOptions = getBuildOptions;

var createBuildTasks = function createBuildTasks(_ref2) {
  var root = _ref2.root,
      packages = _ref2.packages,
      typescript = _ref2.typescript,
      babel = _ref2.babel,
      types = _ref2.types;

  var buildTasks = function buildTasks() {
    var _ref3 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        _ref3$doBabel = _ref3.doBabel,
        doBabel = _ref3$doBabel === void 0 ? true : _ref3$doBabel,
        _ref3$doTs = _ref3.doTs,
        doTs = _ref3$doTs === void 0 ? true : _ref3$doTs,
        _ref3$doTypes = _ref3.doTypes,
        doTypes = _ref3$doTypes === void 0 ? true : _ref3$doTypes;

    var series = [];
    var parallel = [];

    var _loop = function _loop(i) {
      var pack = packages[i];

      var _getBuildOptions = getBuildOptions({
        root: root,
        pack: pack,
        doBabel: doBabel,
        doTs: doTs,
        doTypes: doTypes
      }),
          exts = _getBuildOptions.exts,
          src = _getBuildOptions.src,
          skip = _getBuildOptions.skip,
          opts = (0, _objectWithoutProperties2["default"])(_getBuildOptions, ["exts", "src", "skip"]);

      var taskNames = (0, _lodash.compact)((0, _lodash.map)(opts, function (v, k) {
        return v ? k : false;
      })).join(':');
      var taskName = "build:".concat(pack, ":").concat(taskNames);
      var _parallel = [];
      var _series = [];

      if (skip) {
        return "continue";
      }

      if (doTs) {
        // typescript should be first to .d.ts files
        // babel outputs real js
        _series.push(typescript(pack, {
          exts: exts
        }));
      }

      if (doBabel) {
        _series.push(babel(pack, {
          exts: exts
        }));
      }

      if (doTypes) {
        _parallel.push(types(pack, {
          src: src
        }));
      }

      _gulp["default"].task(taskName, _gulp["default"].parallel.apply(_gulp["default"], [_gulp["default"].series.apply(_gulp["default"], _series)].concat(_parallel)));

      if (opts.doSeries) {
        if (opts.seriesPosition === 'head') {
          series = [taskName].concat(series);
          debug("head ".concat(taskName));
        } else if (opts.dependsOn) {
          var index = series.findIndex(function (s) {
            return s.includes(opts.dependsOn);
          });
          series.splice(index + 1, 0, taskName);
          debug("inserted ".concat(taskName, " at index ").concat(index));
        } else {
          series.push(taskName);
          debug("pushed ".concat(taskName));
        }

        debug({
          series: series
        });
        return "continue";
      }

      parallel.push(taskName);
    };

    for (var i = 0; i < packages.length; i++) {
      var _ret = _loop(i);

      if (_ret === "continue") continue;
    }

    return {
      series: series,
      parallel: parallel
    };
  };

  return gulpBuildTask.bind(undefined, buildTasks);
};

var gulpBuildTask = function gulpBuildTask(buildTasks, taskName) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var tasks = buildTasks(_objectSpread({}, opts));
  debug(function () {
    return tasks;
  });

  _gulp["default"].task.apply(_gulp["default"], [taskName].concat((0, _toConsumableArray2["default"])(flattenTasks(_gulp["default"].series, [].concat((0, _toConsumableArray2["default"])(tasks.series), (0, _toConsumableArray2["default"])(flattenTasks(_gulp["default"].parallel, tasks.parallel)))))));
}; // work-around undertaker complaining about an empty set of tasks


exports.gulpBuildTask = gulpBuildTask;

var flattenTasks = function flattenTasks(takerFn, tasks) {
  if (!tasks.length) return tasks;
  return [takerFn.apply(void 0, (0, _toConsumableArray2["default"])(tasks))];
};

exports.flattenTasks = flattenTasks;
var _default = createBuildTasks;
exports["default"] = _default;